;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "man/entity.h.s"
.include "man/maps.h.s"
.include "man/game.h.s"

.include "cmp/macros_entity.h.s"

.include "sys/collisions.h.s"
.include "sys/input.h.s"
.include "sys/physics.h.s"
.include "sys/render.h.s"

.include "assets/assets.h.s"
.include "cpctelera.h.s"

.include "main.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.module Collisions_System
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_COLLISIONS_INIT
;;    Initializes the collisions system
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_init::
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_COLLISIONS_UPDATE_ONE_ENTITY
;;    Checks collisions of a Collider entity with other ones
;; INPUT:
;;		IX: Pointer to Collider Entity, it is not the end of the array
;; STACK USE: 2 bytes
;; DESTROYS: HL, AF, DE, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_update_one_entity:
   push  ix                   ;;/ Entity with collider type
   pop   iy                   ;;\ IY = IX 

   call  man_entity_getArray  ;; IX = MAIN CHARACTER

   call  sys_check_collisions ;; check collisions between IX and IY

   ;;----------------------------------------------------
   ;; Recover IX
   ;;
   push  iy
   pop   ix
   
   ;;----------------------------------------------------
   ;; HL = applied function, we exchange hl and de, so
   ;; not to lose the value of hl later.
   ;; Recover stored way_back(de) into hl, then they will
   ;; be exchanged in way_back.
   ;;
   ex    de, hl               ;; DE = applied function
   pop   hl                   ;; HL = way_back
   jp    (hl)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_COLLISIONS_UPDATE
;;    Updates all Collider entities
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_update::
   ;;----------------------------------------------------
   ;; Check collisions with tiles
   ;;
   call  sys_collisions_update_tiles

   ;;----------------------------------------------------
   ;; Update all Collider 
   ;; entities with other objects
   ;;
   ld    c, #e_type_collider_mask
   ld    hl, #sys_collisions_update_one_entity
   call  man_entity_forall_matching_types
   
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_CHECK_COLLISIONS
;;    Check if entity_1 has collided with entity_2
;; INPUTS:
;;    IX: Pointer to the first entity (player)
;;    IY: Pointer to the second entity (collider)
;; DESTROYS: AF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_check_collisions::
   ;;----------------------------------------------------
   ;; Check on the X axis
   ;;
   ;;----------------------------------------------------
   ;; NO COLLISION between A(end of the left entity IX)
   ;;                  and B(start of the right entity IY)
   ;;    If (A < B) 
   ;;    If (e_x(ix) + e_w(ix) < e_x(iy))    
   ;;    If (e_x(ix) + e_w(ix) - e_x(iy) < 0)
   ;;
   ld    a, e_x(ix)
   add   e_w(ix)
   sub   e_x(iy)
   ;;----------------------------------------------------
   ;; Work on flag C cause it has the carry of 
   ;; the operations
   ;;
   jp    c, _no_collision 

   ;;----------------------------------------------------
   ;; NO COLLISION between C(end of the left entity IY)
   ;;                  and D(start of the right entity IX)
   ;;    If (C < D) 
   ;;    If (e_x(iy) + e_w(iy) < e_x(ix))    
   ;;    If (e_x(iy) + e_w(iy) - e_x(ix) < 0)
   ;;
   ld    a, e_x(iy)
   add   e_w(iy)
   sub   e_x(ix)
   jp    c, _no_collision

   ;;----------------------------------------------------
   ;; Check on the Y axis
   ;;
   ld    a, e_y(ix)
   add   e_h(ix)
   sub   e_y(iy)
   jp    c, _no_collision

   ld    a, e_y(iy)
   add   e_h(iy)
   sub   e_y(ix)
   jp    c, _no_collision

candy_check:
   ld    a, e_collider(iy)
   ld    c, #e_collider_candy
   cp    c
   jp    z, grabCandy

door_check:
   ld    a, e_collider(iy)
   ld    c, #e_collider_door
   cp    c
   jp    z, _open_door

pumpkin_check:
   ld    a, e_collider(iy)
   ld    c, #e_collider_enemy
   cp    c
   jp    z, pumpkinCollide   

_no_collision:
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; IX = MAIN CHARACTER
;; IY = ENTITY TO CHECK
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pumpkinCollide:
   ;;----------------------------------------------------
   ;; Change game status to dead
   ;;
   ld    hl,   #_game_status
   ld    (hl), #2
ret

grabCandy:
   ;;----------------------------------------------------
   ;; Set for destruction candy entity and set game 
   ;; status to open door.
   ;; man_entity_set4destruction receives entity
   ;; to destroy in IX
   ;;
   push  ix
   push  iy
   pop   ix
   call  man_entity_set4destruction
   push  ix
   pop   iy
   pop   ix
   push  hl
   ld    hl, #_game_status
   ld    (hl), #3             ;; 3 is open door
   pop   hl
ret

_open_door:
   ;;----------------------------------------------------
   ;; Set _check_door to be checked
   ;;
   ld    hl, #_check_door
   ld    (hl), #1
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_COLLISIONS_UPDATE_TILES
;;	   Check collisions with tiles
;; DESTROYS: AF, BC, DE, HL, IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_update_tiles::
   ;;----------------------------------------------------
   ;; IX = Address of Array Entities (first pos = player)
   ;;
   call  man_entity_getArray

   call  sys_collisions_update_up_left
   call  sys_collisions_update_up_right

   call  sys_collisions_update_down_right
   call  sys_collisions_update_down_left

   call  sys_collisions_update_middle_left
   call  sys_collisions_update_middle_right
   
   call  sys_collisions_update_bounding_box_left
   call  sys_collisions_update_bounding_box_right
   

   call  sys_collisions_update_down_middle

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SYS_COLLISIONS_UPDATE_UP_LEFT
;;    Check collisions with upper and left borders
;; INPUTS:
;;    IX: Pointer to the first entity
;; RETURNS: 
;;    
;; DESTROYS: AF, BC, DE, HL, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_collisions_update_up_left:
   call  calculate_address_of_tile_where_the_player_is_iy

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision

   ret

sys_collisions_update_down_right:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite

   xor   a
   cp    (iy)
   ret   z

   call  sys_physics_undo_collision_axis_Y

   call  sys_physics_no_gravity

   ld    hl, #playerStatus
   ld    (hl), #1

   ret

sys_collisions_update_up_right:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_up_right

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision

   ret

sys_collisions_update_down_left:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_down_left

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   call  sys_physics_undo_collision_axis_Y
   call  sys_physics_no_gravity

   ld    hl, #playerStatus
   ld    (hl), #1
   
   ret

sys_collisions_update_down_middle:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_down_middle

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   call  sys_physics_undo_collision_axis_Y
   
   call  sys_physics_no_gravity

   ld    hl, #playerStatus
   ld    (hl), #1

   ret

sys_collisions_update_middle_left:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_left_middle

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision

   ret

sys_collisions_update_middle_right:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_right_middle

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision
   ret

sys_collisions_update_bounding_box_left:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_bound_left

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision_axis_X
   ret

sys_collisions_update_bounding_box_right:
   call  calculate_address_of_tile_where_the_player_is_iy_limit_sprite_bound_right

   ;;----------------------------------------------------
   ;; If player is on tile 0(background)
   ;; then return 
   ;;
   xor   a
   cp    (iy)
   ret   z

   ;;----------------------------------------------------
   ;; If player collided the border
   ;; then undo collision(step back)
   ;;
   call  sys_physics_undo_collision_axis_X
   
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CALCULATE_ADDRESS_OF_TILE_WHERE_THE_PLAYER_IS_IY
;;    Calculates value of tile where the player is.
;; RETURNS: 
;;    IY: Value of the tile
;; DESTROYS: AF, BC, DE, HL, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculate_address_of_tile_where_the_player_is_iy:
   ;;----------------------------------------------------
   ;; MemPtr = Base + Y*20 + X
   ;;         |    | | offset |
   ;; Base is the direction of an array that contains
   ;; the map and offset is how many bytes i have 
   ;; to shift to reach the point i need.
   ;; PixelPosX = 4*TileX
   ;; PixelPosY = 8*TileY
   ;; 
   ;;----------------------------------------------------
   ;; Step 1: Calculate Entity.x / 4
   ;;
   call  man_maps_get_current_de
   ld__iy_de

   ld    a, e_x(ix)     ;; A = Entity.x
   srl   a              ;; A = Entity.x / 2 
   srl   a              ;; A = Entity.x / 4
   ld    c, a           ;;/ BC = Entity.x / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate (Entity.y / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   and   #0b11111000    ;; A = 8*int(Entity.y / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int(Entity.y / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int(Entity.y / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int(Entity.y / 8)
   add   hl, de         ;; HL = 20*int(Entity.y / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate (Entity.y / 8)*20 + (Entity.x / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = (Entity.y / 8)*20 + (Entity.x / 4) and can be
   ;; added to IY
   ;;
   ex    de, hl

   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CALCULATE_ADDRESS_OF_TILE_WHERE_THE_PLAYER_IS_IY_LIMIT_SPRITE
;;    Calculates value of tile where the player ends.
;; RETURNS: 
;;    IY: Value of the tile
;; DESTROYS: AF, BC, DE, HL, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculate_address_of_tile_where_the_player_is_iy_limit_sprite:
   ;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   ;;
   call  man_maps_get_current_de
   ld__iy_de

   ld    a, e_x(ix)     ;; A = Entity.x
   add   e_w(ix)        ;; A = Entity.x + Entity.w
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   add   e_h(ix)        ;; A = Entity.y + Entity.h
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CALCULATE_ADDRESS_OF_TILE_WHERE_THE_PLAYER_IS_IY_LIMIT_SPRITE
;;    Calculates value of tile where the player ends.
;; RETURNS: 
;;    IY: Value of the tile
;; DESTROYS: AF, BC, DE, HL, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculate_address_of_tile_where_the_player_is_iy_limit_sprite_down_left:
   ;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   ;;

   call  man_maps_get_current_de
   ld__iy_de

   ld    a, e_x(ix)     ;; A = Entity.x
   ;; add   e_w(ix)        ;; A = Entity.x + Entity.w
   ;;dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   add   e_h(ix)        ;; A = Entity.y + Entity.h
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CALCULATE_ADDRESS_OF_TILE_WHERE_THE_PLAYER_IS_IY_LIMIT_SPRITE
;;    Calculates value of tile where the player ends.
;; RETURNS: 
;;    IY: Value of the tile
;; DESTROYS: AF, BC, DE, HL, IY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
calculate_address_of_tile_where_the_player_is_iy_limit_sprite_up_right:
   ;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   add   e_w(ix)        ;; A = Entity.x + Entity.w
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ;;add   e_h(ix)        ;; A = Entity.y + Entity.h
   ;;dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret


calculate_address_of_tile_where_the_player_is_iy_limit_sprite_left_middle:
   ;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   ;;add   e_w(ix)        ;; A = Entity.x + Entity.w
   ;;dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ld    b, e_h(ix)
   srl   b
   add   b              ;; A = Entity.y + Entity.h / 2
   ld    b, #0
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

calculate_address_of_tile_where_the_player_is_iy_limit_sprite_right_middle:

;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   add   e_w(ix)        ;; A = Entity.x + Entity.w
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ld    b, e_h(ix)
   srl   b
   add   b       ;; A = Entity.y + Entity.h / 2
   ld    b, #0
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

calculate_address_of_tile_where_the_player_is_iy_limit_sprite_down_middle:

;;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   ld    b, e_w(ix)
   srl   b
   add   b
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ;;ld    b, e_h(ix)     ;; B = Entity.h
   ;;srl b                ;; B = B / 2
   ;;add   b              ;; A = A + B
   ;;ld b, #0
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

calculate_address_of_tile_where_the_player_is_iy_limit_sprite_bound_left:

   ;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   ;;ld    b, e_w(ix)
   ;;srl   b
   ;;add   b
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ld    b, e_h(ix)     ;; B = Entity.h
   ;;srl b 
   dec   b               ;; B = B / 2
   add   b              ;; A = A + B
   ld    b, #0
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret

calculate_address_of_tile_where_the_player_is_iy_limit_sprite_bound_right:

;----------------------------------------------------
   ;; Step 1: Calculate (Entity.x + Entity.w - 1) / 4
   ;; Substract 1, so we check collision with inside tile
   ;; of the player and not where sprite is already ended
   
   call  man_maps_get_current_de
   ld__iy_de
   
   ;;
   ld    a, e_x(ix)     ;; A = Entity.x
   ld    b, e_w(ix)
   ;;srl   b
   add   b
   dec   a              ;; A = Entity.x + Entity.w - 1
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 2 
   srl   a              ;; A = (Entity.x + Entity.w - 1) / 4
   ld    c, a           ;;/ BC = (Entity.x + Entity.w - 1) / 4
   ld    b, #0          ;;\ BC can be added to HL

   ;;----------------------------------------------------
   ;; Step 2: Calculate ((Entity.y + Entity.h - 1) / 8)*20
   ;;
   ld    a, e_y(ix)     ;; A = Entity.y
   ld    b, e_h(ix)     ;; B = Entity.h
   ;;srl b 
   dec   b               ;; B = B / 2
   add   b              ;; A = A + B
   ld    b, #0
   dec   a              ;; A = Entity.y + Entity.h -1
   and   #0b11111000    ;; A = 8*int((Entity.y + Entity.h - 1) / 8)   
   
   ld    e, a           ;;/
   srl   e              ;;| DE = 4*int((Entity.y + Entity.h - 1) / 8)
   ld    d, b           ;;\ D = 0
   
   ld    l, a           ;;/ HL = A = 8*int((Entity.y + Entity.h - 1) / 8) 
   ld    h, d           ;;\
    
   add   hl, hl         ;; HL = 16*int((Entity.y + Entity.h - 1) / 8)
   add   hl, de         ;; HL = 20*int((Entity.y + Entity.h - 1) / 8)

   ;;----------------------------------------------------
   ;; Step 3: Calculate 
   ;; ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4)
   ;;
   add   hl, bc

   ;;----------------------------------------------------
   ;; DE = ((Entity.y + Entity.h - 1) / 8)*20 + 
   ;;             ((Entity.x + Entity.w - 1) / 4) 
   ;; and can be added to IY
   ;;
   ex    de, hl
   
   ;;----------------------------------------------------
   ;; Step 5: Add base to the rest of the formula
   ;;
   add   iy, de

   ret
